import { create } from 'zustand';
import { immer } from 'zustand/middleware/immer';

interface AssetEventSchema {
    modelUuid: string;
    modelName: string;
    position: [number, number, number];
    rotation: [number, number, number];
    state: "idle" | "running" | "stopped" | "disabled" | "error";
}

interface TriggerSchema {
    triggerUuid: string;
    triggerName: string;
    triggerType: "onComplete" | "onStart" | "onStop" | "delay" | "onError";
    delay: number;
    triggeredAsset: {
        triggeredModel: { modelName: string, modelUuid: string };
        triggeredAction: { actionName: string, actionUuid: string };
    } | null;
}

interface TransferPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "default" | "spawn" | "swap" | "despawn";
        material: string | "inherit";
        delay: number | "inherit";
        spawnInterval: number | "inherit";
        spawnCount: number | "inherit";
        triggers: TriggerSchema[];
    }[];
}

interface VehiclePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "travel";
        material: string;
        unLoadDuration: number;
        loadCapacity: number;
        pickUpPoint: { x: number; y: number, z: number } | {};
        unLoadPoint: { x: number; y: number, z: number } | {};
        triggers: TriggerSchema[];
    }[];
}

interface RoboticArmPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "pickAndPlace";
        process: { startPoint: string; endPoint: string };
        triggers: TriggerSchema[];
    }[];
}

interface MachinePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "process";
        processTime: number;
        swapMaterial: string;
        triggers: TriggerSchema[];
    }[];
}

interface StoragePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "storage";
        materials: { materialName: string; materialId: string; quantity: number }[];
        storageCapacity: number;
    }[];
}

interface TransferEventSchema extends AssetEventSchema {
    type: "transfer";
    speed: number;
    points: TransferPointSchema[];
}

interface VehicleSchemaEvent extends AssetEventSchema {
    type: "vehicle";
    speed: number;
    point: VehiclePointSchema;
}

interface RoboticArmSchemaEvent extends AssetEventSchema {
    type: "roboticArm";
    speed: number;
    point: RoboticArmPointSchema;
}

interface MachineSchemaEvent extends AssetEventSchema {
    type: "machine";
    point: MachinePointSchema;
}

interface StorageSchemaEvent extends AssetEventSchema {
    type: "storageUnit";
    point: StoragePointSchema;
}

type EventsSchema = TransferEventSchema | VehicleSchemaEvent | RoboticArmSchemaEvent | MachineSchemaEvent | StorageSchemaEvent | [];

type productsSchema = {
    productName: string;
    productId: string;
    eventsData: EventsSchema[];
}[]

type Store = {
    products: productsSchema;

    // Product-level actions
    addProduct: (productName: string, productId: string) => void;
    removeProduct: (productId: string) => void;
    updateProduct: (productId: string, updates: Partial<{ productName: string; eventsData: EventsSchema[] }>) => void;

    // Event-level actions
    addEventToProduct: (productId: string, event: EventsSchema) => void;
    removeEventFromProduct: (productId: string, modelUuid: string) => void;
    updateEventInProduct: (productId: string, modelUuid: string, updates: Partial<EventsSchema>) => void;

    // Point-level actions
    addPointToEvent: (productId: string, modelUuid: string, point: TransferPointSchema | VehiclePointSchema | RoboticArmPointSchema | MachinePointSchema | StoragePointSchema) => void;
    removePointFromEvent: (productId: string, modelUuid: string, pointUuid: string) => void;
    updatePointInEvent: (
        productId: string,
        modelUuid: string,
        pointUuid: string,
        updates: Partial<TransferPointSchema | VehiclePointSchema | RoboticArmPointSchema | MachinePointSchema | StoragePointSchema>
    ) => void;

    // Action-level actions
    addAction: (
        productId: string,
        modelUuid: string,
        pointUuid: string,
        action: TransferPointSchema['actions'][0] | VehiclePointSchema['actions'][0] | RoboticArmPointSchema['actions'][0] | MachinePointSchema['actions'][0] | StoragePointSchema['actions'][0]
    ) => void;
    removeAction: (actionUuid: string) => void;
    updateAction: (
        actionUuid: string,
        updates: Partial<TransferPointSchema['actions'][0] | VehiclePointSchema['actions'][0] | RoboticArmPointSchema['actions'][0] | MachinePointSchema['actions'][0] | StoragePointSchema['actions'][0]>
    ) => void;

    // Trigger-level actions
    addTrigger: (
        actionUuid: string,
        trigger: TriggerSchema
    ) => void;
    removeTrigger: (triggerUuid: string) => void;
    updateTrigger: (
        triggerUuid: string,
        updates: Partial<TriggerSchema>
    ) => void;

    // Helper functions
    getProductById: (productId: string) => { productName: string; productId: string; eventsData: EventsSchema[] } | undefined;
    getEventByModelUuid: (productId: string, modelUuid: string) => EventsSchema | undefined;
    getPointByUuid: (productId: string, modelUuid: string, pointUuid: string) => TransferPointSchema | VehiclePointSchema | RoboticArmPointSchema | MachinePointSchema | StoragePointSchema | undefined;
    getActionByUuid: (actionUuid: string) => (TransferPointSchema['actions'][0] | VehiclePointSchema['actions'][0] | RoboticArmPointSchema['actions'][0] | MachinePointSchema['actions'][0] | StoragePointSchema['actions'][0]) | undefined;
    getTriggerByUuid: (triggerUuid: string) => TriggerSchema | undefined;
};

export const useProductStore = create<Store>()(
    immer((set, get) => ({
        products: [],
        activeProductId: null,

        // Product-level actions
        addProduct: (productName, productId) => {
            set((state) => {
                const newProduct = {
                    productName,
                    productId: productId,
                    eventsData: []
                };
                state.products.push(newProduct);
            });
        },

        removeProduct: (productId) => {
            set((state) => {
                state.products = state.products.filter(p => p.productId !== productId);
            });
        },

        updateProduct: (productId, updates) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    Object.assign(product, updates);
                }
            });
        },

        // Event-level actions
        addEventToProduct: (productId, event) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    product.eventsData.push(event);
                }
            });
        },

        removeEventFromProduct: (productId, modelUuid) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    product.eventsData = product.eventsData.filter(e => 'modelUuid' in e && e.modelUuid !== modelUuid);
                }
            });
        },

        updateEventInProduct: (productId, modelUuid, updates) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                    if (event) {
                        Object.assign(event, updates);
                    }
                }
            });
        },

        // Point-level actions
        addPointToEvent: (productId, modelUuid, point) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                    if (event && 'points' in event) {
                        (event as TransferEventSchema).points.push(point as TransferPointSchema);
                    } else if (event && 'point' in event) {
                        (event as VehicleSchemaEvent | RoboticArmSchemaEvent | MachineSchemaEvent | StorageSchemaEvent).point = point as any;
                    }
                }
            });
        },

        removePointFromEvent: (productId, modelUuid, pointUuid) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                    if (event && 'points' in event) {
                        (event as TransferEventSchema).points = (event as TransferEventSchema).points.filter(p => p.uuid !== pointUuid);
                    } else if (event && 'point' in event && (event as any).point.uuid === pointUuid) {
                        // For events with single point, we can't remove it, only reset to empty
                        // You might want to handle this differently
                    }
                }
            });
        },

        updatePointInEvent: (productId, modelUuid, pointUuid, updates) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                    if (event && 'points' in event) {
                        const point = (event as TransferEventSchema).points.find(p => p.uuid === pointUuid);
                        if (point) {
                            Object.assign(point, updates);
                        }
                    } else if (event && 'point' in event && (event as any).point.uuid === pointUuid) {
                        Object.assign((event as any).point, updates);
                    }
                }
            });
        },

        // Action-level actions
        addAction: (productId, modelUuid, pointUuid, action) => {
            set((state) => {
                const product = state.products.find(p => p.productId === productId);
                if (product) {
                    const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                    if (event && 'points' in event) {
                        const point = (event as TransferEventSchema).points.find(p => p.uuid === pointUuid);
                        if (point) {
                            point.actions.push(action as any);
                        }
                    } else if (event && 'point' in event && (event as any).point.uuid === pointUuid) {
                        (event as any).point.actions.push(action);
                    }
                }
            });
        },

        removeAction: (actionUuid) => {
            set((state) => {
                for (const product of state.products) {
                    for (const event of product.eventsData) {
                        if ('points' in event) {
                            for (const point of (event as TransferEventSchema).points) {
                                point.actions = point.actions.filter(a => a.actionUuid !== actionUuid);
                            }
                        } else if ('point' in event) {
                            (event as any).point.actions = (event as any).point.actions.filter((a: any) => a.actionUuid !== actionUuid);
                        }
                    }
                }
            });
        },

        updateAction: (actionUuid, updates) => {
            set((state) => {
                for (const product of state.products) {
                    for (const event of product.eventsData) {
                        if ('points' in event) {
                            for (const point of (event as TransferEventSchema).points) {
                                const action = point.actions.find(a => a.actionUuid === actionUuid);
                                if (action) {
                                    Object.assign(action, updates);
                                    return;
                                }
                            }
                        } else if ('point' in event) {
                            const action = (event as any).point.actions.find((a: any) => a.actionUuid === actionUuid);
                            if (action) {
                                Object.assign(action, updates);
                                return;
                            }
                        }
                    }
                }
            });
        },

        // Trigger-level actions
        addTrigger: (actionUuid, trigger) => {
            set((state) => {
                for (const product of state.products) {
                    for (const event of product.eventsData) {
                        if ('points' in event) {
                            for (const point of (event as TransferEventSchema).points) {
                                const action = point.actions.find(a => a.actionUuid === actionUuid);
                                if (action && 'triggers' in action) {
                                    action.triggers.push(trigger);
                                    return;
                                }
                            }
                        } else if ('point' in event) {
                            const action = (event as any).point.actions.find((a: any) => a.actionUuid === actionUuid);
                            if (action && 'triggers' in action) {
                                action.triggers.push(trigger);
                                return;
                            }
                        }
                    }
                }
            });
        },

        removeTrigger: (triggerUuid) => {
            set((state) => {
                for (const product of state.products) {
                    for (const event of product.eventsData) {
                        if ('points' in event) {
                            for (const point of (event as TransferEventSchema).points) {
                                for (const action of point.actions) {
                                    if ('triggers' in action) {
                                        action.triggers = action.triggers.filter(t => t.triggerUuid !== triggerUuid);
                                    }
                                }
                            }
                        } else if ('point' in event) {
                            for (const action of (event as any).point.actions) {
                                if ('triggers' in action) {
                                    action.triggers = action.triggers.filter((t: any) => t.triggerUuid !== triggerUuid);
                                }
                            }
                        }
                    }
                }
            });
        },

        updateTrigger: (triggerUuid, updates) => {
            set((state) => {
                for (const product of state.products) {
                    for (const event of product.eventsData) {
                        if ('points' in event) {
                            for (const point of (event as TransferEventSchema).points) {
                                for (const action of point.actions) {
                                    if ('triggers' in action) {
                                        const trigger = action.triggers.find(t => t.triggerUuid === triggerUuid);
                                        if (trigger) {
                                            Object.assign(trigger, updates);
                                            return;
                                        }
                                    }
                                }
                            }
                        } else if ('point' in event) {
                            for (const action of (event as any).point.actions) {
                                if ('triggers' in action) {
                                    const trigger = action.triggers.find((t: any) => t.triggerUuid === triggerUuid);
                                    if (trigger) {
                                        Object.assign(trigger, updates);
                                        return;
                                    }
                                }
                            }
                        }
                    }
                }
            });
        },

        // Helper functions
        getProductById: (productId) => {
            return get().products.find(p => p.productId === productId);
        },

        getEventByModelUuid: (productId, modelUuid) => {
            const product = get().products.find(p => p.productId === productId);
            if (product) {
                return product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
            }
            return undefined;
        },

        getPointByUuid: (productId, modelUuid, pointUuid) => {
            const product = get().products.find(p => p.productId === productId);
            if (product) {
                const event = product.eventsData.find(e => 'modelUuid' in e && e.modelUuid === modelUuid);
                if (event && 'points' in event) {
                    return (event as TransferEventSchema).points.find(p => p.uuid === pointUuid);
                } else if (event && 'point' in event && (event as any).point.uuid === pointUuid) {
                    return (event as any).point;
                }
            }
            return undefined;
        },

        getActionByUuid: (actionUuid) => {
            const state = get();
            for (const product of state.products) {
                for (const event of product.eventsData) {
                    if ('points' in event) {
                        for (const point of (event as TransferEventSchema).points) {
                            const action = point.actions.find(a => a.actionUuid === actionUuid);
                            if (action) return action;
                        }
                    } else if ('point' in event) {
                        const action = (event as any).point.actions.find((a: any) => a.actionUuid === actionUuid);
                        if (action) return action;
                    }
                }
            }
            return undefined;
        },

        getTriggerByUuid: (triggerUuid) => {
            const state = get();
            for (const product of state.products) {
                for (const event of product.eventsData) {
                    if ('points' in event) {
                        for (const point of (event as TransferEventSchema).points) {
                            for (const action of point.actions) {
                                if ('triggers' in action) {
                                    const trigger = action.triggers.find(t => t.triggerUuid === triggerUuid);
                                    if (trigger) return trigger;
                                }
                            }
                        }
                    } else if ('point' in event) {
                        for (const action of (event as any).point.actions) {
                            if ('triggers' in action) {
                                const trigger = action.triggers.find((t: any) => t.triggerUuid === triggerUuid);
                                if (trigger) return trigger;
                            }
                        }
                    }
                }
            }
            return undefined;
        }
    }))
);